#include "mex.h"
#include <iostream>
#include <fstream>

int imH, imW;
int nbCenter;
int blockRadius;
int offsetX, offsetY;

template<class T>
inline T GET_ELEM2(T* mat, int i, int j, int dim0)
{
	return mat[i + j * dim0];
}

template<class T>
inline void SET_ELEM2(T* mat, int i, int j, int dim0, T value)
{
	mat[i + j * dim0] = value;
}

void getPatch(double* im, double* cxlist, double* cylist, double* patches)
{
	int blockSize = blockRadius * blockRadius;
	for(int i=0; i<nbCenter; i++)
	{
		int nCount = 0;
		int minx = cxlist[i] - offsetX;
		int maxx = minx + blockRadius;
		int miny = cylist[i] - offsetY;
		int maxy = miny + blockRadius;
		for(int x = minx; x < maxx; ++x)
		{
			for (int y = miny; y < maxy; ++y)
			{
				SET_ELEM2(patches, nCount, i, blockSize, GET_ELEM2(im, y, x, imH));
				++nCount;
			}
		}
	}
}


// getPatch(): get patches on image
// 
// patches = getPatch(im, centerlistX, centerlistY, blockSize)
//     Input:
//       im: input image, one chaneel, double
//       centerlistX:
//		 centerlistY: the x,y coor of ith center are centerlistX(i)
//		 blockSize: height of block (square)
//       offset: the offset from the block center to the left-up corner
//     Output:
//       patches: patches(:,i) is the ith patches
void mexFunction( int nlhs, mxArray *plhs[],
				 int nrhs, const mxArray *prhs[] )
{
	/* Check for proper number of arguments. */
	if(nrhs!=5) {
		mexErrMsgTxt("Five input required.");
	} else if(nlhs>1) {
		mexErrMsgTxt("Too many output arguments.");
	}

	/* The input must be a noncomplex scalar double.*/
	if(!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]))
		mexErrMsgTxt("Invalid 1st input!");
	if(!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]))
		mexErrMsgTxt("Invalid 2nd input!");
	if(!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]))
		mexErrMsgTxt("Invalid 3rd input!");
	if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]))
		mexErrMsgTxt("Invalid 4th input!");
	if(!mxIsDouble(prhs[4]) || mxIsComplex(prhs[4]))
		mexErrMsgTxt("Invalid 5th input!");

	/* Assign pointers to each input and output. */
	double* im = mxGetPr(prhs[0]);
	imH = mxGetM(prhs[0]);
	imW = mxGetN(prhs[0]);
	double* cx = mxGetPr(prhs[1]);
	double* cy = mxGetPr(prhs[2]);
	blockRadius = (int)(mxGetScalar(prhs[3]));
	nbCenter = mxGetN(prhs[1]);
	double* offset = mxGetPr(prhs[4]);
	offsetX = (int)(offset[0]);
	offsetY = (int)(offset[1]);	

	/* Create matrix for the return argument. */
	plhs[0] = mxCreateDoubleMatrix(blockRadius*blockRadius, nbCenter, mxREAL);
	double* patches = mxGetPr(plhs[0]);
	
	/* Run */
	getPatch(im, cx, cy, patches);
	
	return;
}


