#include "mex.h"
#include <iostream>
#include <fstream>
#include "math.h"


double* hdimR;
double* hdFeature;
double* hCornerX;
double* hCornerY;
int hblockSize;
double* ldim;
double* lCornerX;
double* lCornerY;
int lblockSize;

int himH;
int himW;
int limH;
int limW;
int nbFeature;

bool scaleif;


template<class T>
inline T GET_ELEM2(T* mat, int i, int j, int dim0)
{
	return mat[i + j * dim0];
}

template<class T>
inline void SET_ELEM2(T* mat, int i, int j, int dim0, T value)
{
	mat[i + j * dim0] = value;
}

void setPatchScale()
{
	// mexPrintf("in setPatchScale\n");

	int** overtime = new int*[himH];
	for(int i=0; i<himH; i++)
		overtime[i] = new int[himW];
	for(int i=0; i<himH; i++)
	{
		for (int j=0; j<himW; j++)
			overtime[i][j] = 0;
	}
	int hlenFeature = hblockSize*hblockSize;
	int llenFeature = lblockSize*lblockSize;

	for(int i=0; i<nbFeature; i++)
	{
		// Get mean & norm
		double origmean = 0.0, origmean2 = 0.0;
		for (int y=static_cast<int>(lCornerY[i]); y<static_cast<int>(lCornerY[i])+lblockSize; y++)
		{
			for (int x=static_cast<int>(lCornerX[i]); x<static_cast<int>(lCornerX[i])+lblockSize; x++)
			{
				double value = GET_ELEM2(ldim, y, x, limH);
				origmean += value;
				origmean2 += value * value;
			}
		}
		origmean = origmean / llenFeature;
		double orignorm = sqrt(origmean2 / llenFeature - origmean * origmean);

		// Get new norm
		// double newmean = 0.0;
		double newmean2 = 0.0;
		for (int j=0; j<hblockSize*hblockSize; j++)
		{
			double value = GET_ELEM2(hdFeature, j, i, hlenFeature);
			// newmean += value;
			newmean2 += value * value;
		}
		// double newmean = newmean/llenFeature;
		double newnorm = sqrt(newmean2 / hlenFeature);
		double scale = ((newnorm == 0.0) ? 0.0 : orignorm/newnorm);

		// Set patch
		for (int yoff=0; yoff<hblockSize; yoff++)
		{
			for (int xoff=0; xoff<hblockSize; xoff++)
			{
				int x = xoff + hCornerX[i];
				int y = yoff + hCornerY[i];
				double depth = GET_ELEM2(hdFeature, xoff*hblockSize+yoff, i, hlenFeature) * scale + origmean;
				depth = (depth > 255.0 ? 255.0 : depth);
				depth = (depth < 0.0 ? 0.0 : depth);
				int tmpOvertime = overtime[y][x];
				if(tmpOvertime == 0)
					SET_ELEM2(hdimR, y, x, himH, depth);
				else
					SET_ELEM2(hdimR, y, x, himH, (GET_ELEM2(hdimR, y, x, himH)*tmpOvertime +
					depth) / (tmpOvertime + 1));
				++overtime[y][x];
			}
		}
	}

	for(int i=0; i<himH; i++)
		delete [] overtime[i];
	delete [] overtime;
}

void setPatchOrig()
{
	// mexPrintf("in setPatchOrig\n");

	int** overtime = new int*[himH];
	for(int i=0; i<himH; i++)
		overtime[i] = new int[himW];
	for(int i=0; i<himH; i++)
	{
		for (int j=0; j<himW; j++)
			overtime[i][j] = 0;
	}
	int hlenFeature = hblockSize*hblockSize;
	int llenFeature = lblockSize*lblockSize;

	for(int i=0; i<nbFeature; i++)
	{
		// Get mean & norm
		double origmean = 0.0;
		for (int y=static_cast<int>(lCornerY[i]); y<static_cast<int>(lCornerY[i])+lblockSize; y++)
		{
			for (int x=static_cast<int>(lCornerX[i]); x<static_cast<int>(lCornerX[i])+lblockSize; x++)
			{
				double value = GET_ELEM2(ldim, y, x, limH);
				origmean += value;
			}
		}
		origmean = origmean / llenFeature;

		// Set patch
		for (int yoff=0; yoff<hblockSize; yoff++)
		{
			for (int xoff=0; xoff<hblockSize; xoff++)
			{
				int x = xoff + hCornerX[i];
				int y = yoff + hCornerY[i];
				double depth = GET_ELEM2(hdFeature, xoff*hblockSize+yoff, i, hlenFeature) + origmean;
				depth = (depth > 255.0 ? 255.0 : depth);
				depth = (depth < 0.0 ? 0.0 : depth);
				int tmpOvertime = overtime[y][x];
				if(tmpOvertime == 0)
					SET_ELEM2(hdimR, y, x, himH, depth);
				else
					SET_ELEM2(hdimR, y, x, himH, (GET_ELEM2(hdimR, y, x, himH)*tmpOvertime +
					depth) / (tmpOvertime + 1));
				++overtime[y][x];
			}
		}
	}

	for(int i=0; i<himH; i++)
		delete [] overtime[i];
	delete [] overtime;
}


// setPatch(): set patches on image
// 
// setPatch(hdimR, hdFeature, hCornerX, hCornerY, hblockSize, ldim, lCornerX, lCornerY, lblockSize, scaleif)
//     Input:
//			hdimR: high resolution depth image to be resolved
//			hdFeature: estiamted hdim feature
//			hCornerX, hCornerY: the up left corner of ith patch in high resolution image
//			hblockSize:
//			ldim: low resolution depth image
//			lCornerX, lCornerY: the up left corner of ith patch in high resolution image
//			lblockSize:
//			scale: whether to scale hdFeature
//     Output:
//
void mexFunction( int nlhs, mxArray *plhs[],
				 int nrhs, const mxArray *prhs[] )
{
	/* Check for proper number of arguments. */
	if(nrhs<9) {
		mexErrMsgTxt("Nine input required.");
	} else if(nlhs>0) {
		mexErrMsgTxt("Too many output arguments.");
	}

	/* The input must be a noncomplex scalar double.*/
	if(!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]))
		mexErrMsgTxt("Invalid 1st input!");
	if(!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]))
		mexErrMsgTxt("Invalid 2nd input!");
	if(!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]))
		mexErrMsgTxt("Invalid 3rd input!");
	if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]))
		mexErrMsgTxt("Invalid 4th input!");
	if(!mxIsDouble(prhs[4]) || mxIsComplex(prhs[4]))
		mexErrMsgTxt("Invalid 5th input!");
	if(!mxIsDouble(prhs[5]) || mxIsComplex(prhs[5]))
		mexErrMsgTxt("Invalid 6nd input!");
	if(!mxIsDouble(prhs[6]) || mxIsComplex(prhs[6]))
		mexErrMsgTxt("Invalid 7rd input!");
	if(!mxIsDouble(prhs[7]) || mxIsComplex(prhs[7]))
		mexErrMsgTxt("Invalid 8th input!");
	if(!mxIsDouble(prhs[8]) || mxIsComplex(prhs[8]))
		mexErrMsgTxt("Invalid 9th input!");

	/* Assign pointers to each input and output. */
	hdimR = mxGetPr(prhs[0]);
	hdFeature = mxGetPr(prhs[1]);
	hCornerX = mxGetPr(prhs[2]);
	hCornerY = mxGetPr(prhs[3]);
	hblockSize = (int)(mxGetScalar(prhs[4]));
	ldim = mxGetPr(prhs[5]);
	lCornerX = mxGetPr(prhs[6]);
	lCornerY = mxGetPr(prhs[7]);
	lblockSize = (int)(mxGetScalar(prhs[8]));

	himH = static_cast<int>(mxGetM(prhs[0]));
	himW = static_cast<int>(mxGetN(prhs[0]));
	limH = static_cast<int>(mxGetM(prhs[5]));
	limW = static_cast<int>(mxGetN(prhs[5]));
	nbFeature = static_cast<int>(mxGetN(prhs[2]));

	if(nrhs == 9)
		scaleif = false;
	else
		scaleif = (((int)mxGetScalar(prhs[9])) == 1);
	
	/* Run */
	if(scaleif)
		setPatchScale();
	else
		setPatchOrig();
	
	return;
}


