function upsample_color1(testList, model, PARA)

% upsample_color1(): SC upsampling
% 
%     Input:
%       testList: 
%           testList.indir - input directory
%           testList.outdir - output directory
%           testList.imageList - imagelist
%       model: model for METest
%       PARA: parameter
%     Output:
%
% Xue Tianfan, <xtf009@ie.cuhk.edu.hk>, MMLab CUHK

addpath .\feature;

vThresh = PARA.vThresh;
overlap = PARA.overlap;
scale = PARA.scale;
blockSize = PARA.blockSize;
hf1 = [-1,0,1];
vf1 = [-1,0,1]';
hf2 = [1,0,-2,0,1];
vf2 = [1,0,-2,0,1]';

nbTest = length(testList.imageList);
tic;
for i=1:nbTest
    cim = rgb2gray(imread(sprintf('%s%s_c.bmp', testList.indir, testList.imageList{i})));
    ldim = imread(sprintf('%s%s_ld.bmp', testList.indir, testList.imageList{i}));
    ihdim = imresize(ldim, scale, 'bicubic');
    hSize = size(cim); hSize = hSize(1:2);
    lSize = size(ldim);
    if sum(lSize * scale == hSize) ~= 2
        throw(MException('VerifyOutput:OutOfBounds', 'Invalid image size'));
    end
    ihdimd1x = conv2(double(ihdim), hf1,'same');
    ihdimd1y = conv2(double(ihdim), vf1,'same'); 
    ihdimd2x = conv2(double(ihdim), hf2,'same');
    ihdimd2y = conv2(double(ihdim), vf2,'same');
    cesb = imfilter(double(cim), fspecial('sobel')) .* ...
         imdilate(edge(ihdim, 'sobel', 0.01), strel('diamond', 1)) / 255;

    % interpolation result
    hdimI = imresize(ldim, scale);
    
    % extract patches
    [cornerX, cornerY] = meshgrid(0:(blockSize-overlap):hSize(2)-blockSize, 0:(blockSize-overlap):hSize(1)-blockSize);
    cornerX = cornerX(:)'; cornerY = cornerY(:)';
    ldFeature = getPatch(double(ihdim), cornerX, cornerY, blockSize, [0,0]);
    selectPatch = var(ldFeature, [], 1) > vThresh^2;
    cornerX = cornerX(:, selectPatch); cornerY = cornerY(:, selectPatch);
    ihddFeature = [getPatch(ihdimd1x, cornerX, cornerY, blockSize, [0,0]);...
        getPatch(ihdimd1y, cornerX, cornerY, blockSize, [0,0]);...
        getPatch(ihdimd2x, cornerX, cornerY, blockSize, [0,0]);...
        getPatch(ihdimd2y, cornerX, cornerY, blockSize, [0,0])];
    ceFeature = getPatch(cesb, cornerX, cornerY, blockSize, [0,0]);
    
    % convert ld to hd
    hdFeature = ScSR(model, [ihddFeature; ceFeature*PARA.colorweight]);
    hdimR = double(hdimI);
    setPatch(hdimR, hdFeature*255, cornerX, cornerY, blockSize, double(ihdim), cornerX, cornerY, blockSize, 1);
    hdimR = uint8(backprojection(hdimR, ldim, PARA.maxIter));

    imwrite(hdimR, sprintf('%s%s_srhd.bmp', testList.outdir, testList.imageList{i}));
end

end









